@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
  * jsonb_gin_ops.c
- *     Support GIN over jsonb with jsquery operation
+ *	Support GIN over jsonb with jsquery operation
  *
  * Copyright (c) 2014, PostgreSQL Global Development Group
  * Author: Alexander Korotkov <aekorotkov@gmail.com>
  *
  * IDENTIFICATION
- *    contrib/jsquery/jsonb_gin_ops.c
+ *	contrib/jsquery/jsonb_gin_ops.c
  *
  *-------------------------------------------------------------------------
  */
@@ -258,51 +258,54 @@ make_gin_key(JsonbValue *v, uint32 hash)
 {
 	GINKey *key;
 
-	if (v->type == jbvNull)
+	switch (v->type)
 	{
-		key = (GINKey *)palloc(GINKEYLEN);
-		key->type = v->type;
-		SET_VARSIZE(key, GINKEYLEN);
-	}
-	else if (v->type == jbvBool)
-	{
-		key = (GINKey *)palloc(GINKEYLEN);
-		key->type = v->type | (v->val.boolean ? GINKeyTrue : 0);
-		SET_VARSIZE(key, GINKEYLEN);
-	}
-	else if (v->type == jbvArray)
-	{
-		key = (GINKey *)palloc(GINKEYLEN);
-		key->type = v->type;
-		if (v->val.array.nElems == 0)
-			key->type |= GINKeyEmptyArray;
-		SET_VARSIZE(key, GINKEYLEN);
-	}
-	else if (v->type == jbvObject)
-	{
-		key = (GINKey *)palloc(GINKEYLEN);
-		key->type = v->type;
-		SET_VARSIZE(key, GINKEYLEN);
-	}
-	else if (v->type == jbvNumeric)
-	{
-		key = (GINKey *)palloc(GINKeyLenNumeric(VARSIZE_ANY(v->val.numeric)));
-		key->type = v->type;
-		memcpy(GINKeyDataNumeric(key), v->val.numeric, VARSIZE_ANY(v->val.numeric));
-		SET_VARSIZE(key, GINKeyLenNumeric(VARSIZE_ANY(v->val.numeric)));
-	}
-	else if (v->type == jbvString)
-	{
-		key = (GINKey *)palloc(GINKeyLenString);
-		key->type = v->type;
-		GINKeyDataString(key) = hash_any((unsigned char *)v->val.string.val,
-														  v->val.string.len);
-		SET_VARSIZE(key, GINKeyLenString);
-	}
-	else
-	{
-		elog(ERROR, "GINKey must be scalar");
+		case jbvNull:
+		case jbvObject:
+		{
+			key = (GINKey *)palloc(GINKEYLEN);
+			key->type = v->type;
+			SET_VARSIZE(key, GINKEYLEN);
+			break;
+		}
+		case jbvBool:
+		{
+			key = (GINKey *)palloc(GINKEYLEN);
+			key->type = v->type | (v->val.boolean ? GINKeyTrue : 0);
+			SET_VARSIZE(key, GINKEYLEN);
+			break;
+		}
+		case jbvArray:
+		{
+			key = (GINKey *)palloc(GINKEYLEN);
+			key->type = v->type;
+			if (v->val.array.nElems == 0)
+				key->type |= GINKeyEmptyArray;
+
+			SET_VARSIZE(key, GINKEYLEN);
+			break;
+		}
+		case jbvNumeric:
+		{
+			key = (GINKey *) palloc0(GINKeyLenNumeric(VARSIZE_ANY(v->val.numeric)));
+			key->type = v->type;
+			memcpy(GINKeyDataNumeric(key), v->val.numeric, VARSIZE_ANY(v->val.numeric));
+			SET_VARSIZE(key, GINKeyLenNumeric(VARSIZE_ANY(v->val.numeric)));
+			break;
+		}
+		case jbvString:
+		{
+			key = (GINKey *) palloc0(GINKeyLenString);
+			key->type = v->type;
+			GINKeyDataString(key) = hash_any((unsigned char *)v->val.string.val,
+															  v->val.string.len);
+			SET_VARSIZE(key, GINKeyLenString);
+			break;
+		}
+		default:
+			elog(ERROR, "GINKey must be scalar");
 	}
+
 	key->hash = hash;
 	return key;
 }
@@ -462,7 +465,7 @@ make_value_path_entry_handler(ExtractedNode *node, Pointer extra)
 {
 	Entries	   *e = (Entries *)extra;
 	uint32		hash;
-	bool		lossy, partialMatch;
+	bool		lossy, partialMatch = false;
 	GINKey	   *key;
 	KeyExtra   *keyExtra;
 	int			result;
@@ -691,6 +694,8 @@ gin_extract_jsonb_value_path_internal(Jsonb *jb, int32 *nentries, uint32 **bloom
 				stack->parent = tmp;
 				break;
 			case WJB_KEY:
+				if (!stack) /* should never happen */
+					elog(ERROR, "error jsonb iteration");
 				stack->hash = 0;
 				JsonbHashScalarValue(&v, &stack->hash);
 				break;
@@ -709,6 +714,8 @@ gin_extract_jsonb_value_path_internal(Jsonb *jb, int32 *nentries, uint32 **bloom
 				break;
 			case WJB_END_OBJECT:
 				/* Pop the stack */
+				if (!stack) /* should never happen */
+					elog(ERROR, "error jsonb iteration");
 				tmp = stack->parent;
 				pfree(stack);
 				stack = tmp;
@@ -728,7 +735,7 @@ gin_extract_jsonb_value_path_internal(Jsonb *jb, int32 *nentries, uint32 **bloom
 Datum
 gin_extract_jsonb_value_path(PG_FUNCTION_ARGS)
 {
-	Jsonb	   *jb = PG_GETARG_JSONB(0);
+	Jsonb	   *jb = PG_GETARG_JSONB_P(0);
 	int32	   *nentries = (int32 *) PG_GETARG_POINTER(1);
 
 	PG_RETURN_POINTER(gin_extract_jsonb_value_path_internal(jb, nentries, NULL));
@@ -766,12 +773,12 @@ gin_extract_jsonb_query_value_path(PG_FUNCTION_ARGS)
 	switch(strategy)
 	{
 		case JsonbContainsStrategyNumber:
-			jb = PG_GETARG_JSONB(0);
+			jb = PG_GETARG_JSONB_P(0);
 			entries = gin_extract_jsonb_value_path_internal(jb, nentries, NULL);
 			break;
 
 		case JsonbNestedContainsStrategyNumber:
-			jb = PG_GETARG_JSONB(0);
+			jb = PG_GETARG_JSONB_P(0);
 			entries = gin_extract_jsonb_value_path_internal(jb, nentries, &bloom);
 
 			n = *nentries;
@@ -863,7 +870,7 @@ gin_triconsistent_jsonb_value_path(PG_FUNCTION_ARGS)
 {
 	GinTernaryValue   *check = (GinTernaryValue *) PG_GETARG_POINTER(0);
 	StrategyNumber strategy = PG_GETARG_UINT16(1);
-	/* Jsonb	   *query = PG_GETARG_JSONB(2); */
+	/* Jsonb	   *query = PG_GETARG_JSONB_P(2); */
 	int32		nkeys = PG_GETARG_INT32(3);
 	Pointer	   *extra_data = (Pointer *) PG_GETARG_POINTER(4);
 	GinTernaryValue	res = GIN_TRUE;
@@ -1109,6 +1116,9 @@ gin_extract_jsonb_path_value_internal(Jsonb *jb, int32 *nentries)
 			entries = (Datum *) repalloc(entries, sizeof(Datum) * total);
 		}
 
+		if (!stack) /* should never happen */
+			elog(ERROR, "error jsonb iteration");
+
 		switch (r)
 		{
 			case WJB_BEGIN_ARRAY:
@@ -1131,6 +1141,8 @@ gin_extract_jsonb_path_value_internal(Jsonb *jb, int32 *nentries)
 				break;
 			case WJB_KEY:
 				/* Initialize hash from parent */
+				if (!stack->parent) /* should never happen */
+					elog(ERROR, "error jsonb iteration");
 				stack->hash = stack->parent->hash;
 				JsonbHashScalarValue(&v, &stack->hash);
 				break;
@@ -1162,7 +1174,7 @@ gin_extract_jsonb_path_value_internal(Jsonb *jb, int32 *nentries)
 Datum
 gin_extract_jsonb_path_value(PG_FUNCTION_ARGS)
 {
-	Jsonb	   *jb = PG_GETARG_JSONB(0);
+	Jsonb	   *jb = PG_GETARG_JSONB_P(0);
 	int32	   *nentries = (int32 *) PG_GETARG_POINTER(1);
 
 	PG_RETURN_POINTER(gin_extract_jsonb_path_value_internal(jb, nentries));
@@ -1175,6 +1187,7 @@ gin_debug_query_path_value(PG_FUNCTION_ARGS)
 	Entries		e = {0};
 	char	   *s;
 
+
 	jq = PG_GETARG_JSQUERY(0);
 	s = debugJsQuery(jq, make_path_value_entry_handler,
 										check_path_value_entry_handler, (Pointer)&e);
@@ -1199,7 +1212,7 @@ gin_extract_jsonb_query_path_value(PG_FUNCTION_ARGS)
 	switch(strategy)
 	{
 		case JsonbContainsStrategyNumber:
-			jb = PG_GETARG_JSONB(0);
+			jb = PG_GETARG_JSONB_P(0);
 			entries = gin_extract_jsonb_path_value_internal(jb, nentries);
 			break;
 
@@ -1240,7 +1253,7 @@ gin_consistent_jsonb_path_value(PG_FUNCTION_ARGS)
 {
 	bool	   *check = (bool *) PG_GETARG_POINTER(0);
 	StrategyNumber strategy = PG_GETARG_UINT16(1);
-	/* Jsonb	   *query = PG_GETARG_JSONB(2); */
+	/* Jsonb	   *query = PG_GETARG_JSONB_P(2); */
 	int32		nkeys = PG_GETARG_INT32(3);
 	Pointer	   *extra_data = (Pointer *) PG_GETARG_POINTER(4);
 	bool	   *recheck = (bool *) PG_GETARG_POINTER(5);
@@ -1281,7 +1294,7 @@ gin_triconsistent_jsonb_path_value(PG_FUNCTION_ARGS)
 {
 	GinTernaryValue *check = (GinTernaryValue *) PG_GETARG_POINTER(0);
 	StrategyNumber strategy = PG_GETARG_UINT16(1);
-	/* Jsonb	   *query = PG_GETARG_JSONB(2); */
+	/* Jsonb	   *query = PG_GETARG_JSONB_P(2); */
 	int32		nkeys = PG_GETARG_INT32(3);
 	Pointer	   *extra_data = (Pointer *) PG_GETARG_POINTER(4);
 	GinTernaryValue res = GIN_TRUE;
