@@ -1,6 +1,7 @@
 CREATE EXTENSION jsquery;
 
 set escape_string_warning=off;
+set standard_conforming_strings=on;
 
 CREATE TABLE test_jsquery (v jsonb);
 
@@ -295,6 +296,38 @@ select '{"a":[1,2]}' @@ '*.@# in (2, 4)'::jsquery;
 select '{"a":[1,2]}' @@ '*.@# ($ = 4 or $ = 2)'::jsquery;
 select '{"a":[1,2]}' @@ '@#  = 1'::jsquery;
 
+--filter
+select '?( not b>0). x'::jsquery;
+select 'a.?(b>0 and x= 0 ) .c'::jsquery;
+select 'a.$. ?(b>0 and x= 0 ) . c.k'::jsquery;
+select 'a.$.? (b>0 and x.*= 0 ).c.k'::jsquery;
+select '[{"a":1, "b":10}, {"a":2, "b":20}, {"a":3, "b":30}]'::jsonb @@ '#. ?(a < 0) (b=20)';
+select '[{"a":1, "b":10}, {"a":2, "b":20}, {"a":3, "b":30}]'::jsonb @@ '#. ?(a > 0) (b=20)';
+select '[{"a":1, "b":10}, {"a":2, "b":20}, {"a":3, "b":30}]'::jsonb @@ '#. ?(a > 1) (b=20)';
+select '[{"a":1, "b":10}, {"a":2, "b":20}, {"a":3, "b":30}]'::jsonb @@ '#. ?(a > 2) (b=20)';
+select '[{"a":1, "b":10}, {"a":2, "b":20}, {"a":3, "b":30}]'::jsonb @@ '#. ?(a > 3) (b=20)';
+select '[{"a":1, "b":10}, {"a":2, "b":20}]'::jsonb ~~ '#.a';
+select '[{"a":1, "b":10}, {"a":2, "b":20}]'::jsonb ~~ '#. ?(a > 1). b';
+select '[{"a":1, "b":10}, {"a":2, "b":20}, {"a":3, "b":30}]'::jsonb ~~ '# . ?(a > 1)';
+select '[{"a":1, "b":10}, {"a":2, "b":20}, {"a":3, "b":30}]'::jsonb ~~ '%';
+select '{"a":1, "b":2, "c":3}'::jsonb ~~ '%';
+select '{"a":1, "b":2, "c":3}'::jsonb ~~ '% . ? ( $ > 2 )';
+select '{"a":1, "b":2, "c":3}'::jsonb ~~ '% . ? ( $ > 2 ).$';
+select '{"a":1, "b":2, "c":3}'::jsonb ~~ '? ( % > 2 )';
+select '{"a":1, "b":2, "c":3}'::jsonb ~~ '? ( %: > 0 )';
+select '{"a":1, "b":2, "c":3}'::jsonb ~~ '? ( %: > 2 )';
+select '[{"a":1, "b":10}, {"a":2, "b":20}, {"a":3, "b":30}]'::jsonb ~~ '#';
+select '[1,2,3]'::jsonb ~~ '#';
+select '[1,2,3]'::jsonb ~~ '#. ?($ > 2)';
+select '[1,2,3]'::jsonb ~~ '#. ?($ > 2).$';
+select '[1,2,3]'::jsonb ~~ ' ?(#.$ > 2).$';
+select '[1,2,3]'::jsonb ~~ ' ?(#:.$ > 2).$';
+select '[1,2,3]'::jsonb ~~ ' ?(#:.$ > 0).$';
+select '{"a": {"b": {"c": 1}}}'::jsonb ~~ '*.?(c >0)';
+select '{"a": {"b": {"c": 1}}}'::jsonb ~~ '?(*.c >0)';
+select '{"tags":[{"term":["NYC", "CYN"]}, {"term":["1NYC", "1CYN"]} ]}'::jsonb ~~ 'tags.#.term.#. ? ( $ = "NYC")';
+select '{"tags":[{"term":["NYC", "CYN"]}, {"term":["1NYC", "1CYN"]} ]}'::jsonb ~~ 'tags.#.term. ? ( # = "NYC")';
+
 --ALL
 select 'a.*: = 4'::jsquery;
 select '%: = 4'::jsquery;
@@ -336,6 +369,30 @@ SELECT 'test.# IN (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,2
 select '[]' @@ '(@# > 0 and #: = 16)'::jsquery;
 select '[16]' @@ '(@# > 0 and #: = 16)'::jsquery;
 
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb @@ 'a.b or b.d';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb @@ 'a.c or b.d';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb @@ 'a.g or b.d';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb @@ 'a.g or b.c';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb @@ 'a.b and b.d';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb @@ 'a.c and b.d';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb @@ 'a.c and b.b';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb @@ 'a.g and b.d';
+
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ 'a.b and b.d';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ 'a.b and b.c';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ 'a.b and (b.d or b.c)';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ 'a.b and (b.c or b.d)';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ 'a.b and a.c and b.d';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ '(a.b or a.c) and b.d';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ '(a.e or a.c) and b.d';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ '(a.e or a.g) and b.d';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ 'a.b or (b.d or b.c)';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ 'b.d or (a.b or a.c)';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ 'b.d or (a.b and a.c)';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ 'b.f or (a.b and a.c)';
+select '{"a": {"b": 1, "c": 2}, "b": {"d":3}}'::jsonb ~~ 'b.d and (a.b and a.c)';
+select '{"a": {"b": [6,5,4], "c": 2}, "b": {"d":3}}'::jsonb ~~ 'b.d and (a.b and a.c)';
+
 --extract entries for index scan
 
 SELECT gin_debug_query_path_value('NOT NOT NOT x(y(NOT (a=1) and NOT (b=2)) OR NOT NOT (c=3)) and z = 5');
@@ -360,6 +417,9 @@ SELECT gin_debug_query_path_value('x is object');
 SELECT gin_debug_query_path_value('#:(x=1) AND %:(y=1) AND *:(z=1)');
 SELECT gin_debug_query_path_value('#:(NOT x=1) AND %:(NOT y=1) AND *:(NOT z=1)');
 SELECT gin_debug_query_path_value('NOT #:(NOT x=1) AND NOT %:(NOT y=1) AND NOT *:(NOT z=1)');
+SELECT gin_debug_query_path_value('$ = true');
+SELECT gin_debug_query_path_value('$ . ? (review_votes > 10) . review_rating < 7');
+SELECT gin_debug_query_path_value('similar_product_ids . ? (# = "B0002W4TL2") . $');
 
 SELECT gin_debug_query_value_path('NOT NOT NOT x(y(NOT (a=1) and NOT (b=2)) OR NOT NOT (c=3)) and z = 5');
 SELECT gin_debug_query_value_path('NOT #(x=1) and NOT *(y=1) and NOT %(z=1) ');
@@ -386,6 +446,10 @@ SELECT gin_debug_query_value_path('#:(NOT x=1) AND %:(NOT y=1) AND *:(NOT z=1)')
 SELECT gin_debug_query_value_path('NOT #:(NOT x=1) AND NOT %:(NOT y=1) AND NOT *:(NOT z=1)');
 SELECT gin_debug_query_value_path('(@# > 0 and #: = 16)');
 SELECT gin_debug_query_value_path('*.@# ($ = 4 or $ = 2)');
+SELECT gin_debug_query_value_path('tags.#.term. ? ( # = "NYC").x > 0');
+SELECT gin_debug_query_value_path('$ = true');
+SELECT gin_debug_query_value_path('$ . ? (review_votes > 10) . review_rating < 7');
+SELECT gin_debug_query_value_path('similar_product_ids . ? (# = "B0002W4TL2") . $');
 
 ---table and index
 
@@ -434,6 +498,11 @@ select count(*) from test_jsquery where v @@ 'similar_product_ids.#: is string';
 select count(*) from test_jsquery where v @@ 'NOT similar_product_ids.#: (NOT $ = "0440180295")';
 select count(*) from test_jsquery where v @@ '$ > 2';
 select count(*) from test_jsquery where v @@ '$ = false';
+select count(*) from test_jsquery where v @@ 't';
+select count(*) from test_jsquery where v @@ '$';
+select count(*) from test_jsquery where v @@ 'similar_product_ids.#';
+select count(*) from test_jsquery where v @@ '$ . ? (review_votes > 10) . review_rating < 7';
+select count(*) from test_jsquery where v @@ 'similar_product_ids . ? (# = "B0002W4TL2") . $';
 
 select v from test_jsquery where v @@ 'array <@ [2,3]' order by v;
 select v from test_jsquery where v @@ 'array && [2,3]' order by v;
@@ -480,6 +549,11 @@ select count(*) from test_jsquery where v @@ 'similar_product_ids.#: is string';
 select count(*) from test_jsquery where v @@ 'NOT similar_product_ids.#: (NOT $ = "0440180295")';
 select count(*) from test_jsquery where v @@ '$ > 2';
 select count(*) from test_jsquery where v @@ '$ = false';
+select count(*) from test_jsquery where v @@ 't';
+select count(*) from test_jsquery where v @@ '$';
+select count(*) from test_jsquery where v @@ 'similar_product_ids.#';
+select count(*) from test_jsquery where v @@ '$ . ? (review_votes > 10) . review_rating < 7';
+select count(*) from test_jsquery where v @@ 'similar_product_ids . ? (# = "B0002W4TL2") . $';
 
 explain (costs off) select v from test_jsquery where v @@ 'array <@ [2,3]' order by v;
 explain (costs off) select v from test_jsquery where v @@ 'array && [2,3]' order by v;
@@ -533,6 +607,11 @@ select count(*) from test_jsquery where v @@ 'similar_product_ids.#: is string';
 select count(*) from test_jsquery where v @@ 'NOT similar_product_ids.#: (NOT $ = "0440180295")';
 select count(*) from test_jsquery where v @@ '$ > 2';
 select count(*) from test_jsquery where v @@ '$ = false';
+select count(*) from test_jsquery where v @@ 't';
+select count(*) from test_jsquery where v @@ '$';
+select count(*) from test_jsquery where v @@ 'similar_product_ids.#';
+select count(*) from test_jsquery where v @@ '$ . ? (review_votes > 10) . review_rating < 7';
+select count(*) from test_jsquery where v @@ 'similar_product_ids . ? (# = "B0002W4TL2") . $';
 
 explain (costs off) select v from test_jsquery where v @@ 'array <@ [2,3]' order by v;
 explain (costs off) select v from test_jsquery where v @@ 'array && [2,3]' order by v;
